import tabulate

FIELDS = {
    "Domain": {
        "Domain": {"field": "domain", "truncate": 0},
        "Active users (30d)": {"field": "usage_users_active_month"},
        "Active users (180d)": {"field": "usage_users_active_half_year"},
        "Users": {"field": "usage_users_total"},
        "Listenings": {"field": "usage_listenings_total"},
        "Downloads": {"field": "usage_downloads_total"},
        "Tracks": {"field": "library_tracks_total"},
        "Albums": {"field": "library_albums_total"},
        "Artists": {"field": "library_artists_total"},
        "Hours of music": {"field": "library_music_hours"},
        "Open registrations": {"field": "open_registrations"},
        "Federation enabled": {"field": "federation_enabled"},
        "Anonymous access": {"field": "anonymous_can_listen"},
        "Private": {"field": "private"},
    },
    "Summary": {
        "Domains": {"field": "domain"},
        "Active users (30d)": {"field": "usage_users_active_month"},
        "Active users (180d)": {"field": "usage_users_active_half_year"},
        "Users": {"field": "usage_users_total"},
        "Listenings": {"field": "usage_listenings_total"},
        "Downloads": {"field": "usage_downloads_total"},
        "Tracks": {"field": "library_tracks_total"},
        "Albums": {"field": "library_albums_total"},
        "Artists": {"field": "library_artists_total"},
        "Hours of music": {"field": "library_music_hours"},
        "Open registrations": {"field": "open_registrations"},
        "Federation enabled": {"field": "federation_enabled"},
        "Anonymous access": {"field": "anonymous_can_listen"},
        "Private": {"field": "private"},
    },
}

TABLE_FORMATS = sorted(tabulate._table_formats.keys())


def get_value(obj, config, truncate=30):
    field = config["field"]
    value = obj[field]

    if config.get("handler"):
        value = config["handler"](value)
    value = str(value)
    if truncate and len(value) > truncate:
        value = value[:truncate] + "…"
    return value


def table(objects, fields, type, headers=True, format="simple", sort=None):
    configs = {}

    if sort:
        reversed = sort.startswith("-")
        sort = sort.lstrip("-")
        sort_field = FIELDS[type][sort]["field"]
        objects = sorted(objects, key=lambda v: v[sort_field] or 0, reverse=reversed)
    for f in fields:
        try:
            configs[f] = FIELDS[type][f]
        except KeyError:
            try:
                configs[f] = FIELDS["Domain"][f]
            except KeyError:
                raise ValueError(f"{f} is not a valid field for type {type}")

    headers = fields if headers else []
    rows = [
        [
            get_value(obj, configs[f], truncate=configs[f].get("truncate", 30))
            for f in fields
        ]
        for obj in objects
    ]
    return tabulate.tabulate(rows, headers=headers, tablefmt=format)


def obj_table(obj, fields, type, headers=True, format="simple"):
    """
    same as table(), but output a two-column table for a single object,
    with fields on the left and values on the right
    """
    configs = {}

    for f in fields:
        try:
            configs[f] = FIELDS[type][f]
        except KeyError:
            try:
                configs[f] = FIELDS["*"][f]
            except KeyError:
                raise ValueError(f"{f} is not a valid field for type {type}")

    rows = [
        (f, get_value(obj, configs[f], truncate=configs[f].get("truncate", 30)))
        for f in fields
    ]

    return tabulate.tabulate(rows, headers=[], tablefmt=format)
