import pytest

from funkwhale_network import schemas


@pytest.mark.parametrize(
    "value, expected",
    [
        (
            "1.2.3-dev+build-1",
            {
                "major": 1,
                "minor": 2,
                "patch": 3,
                "prerelease": "dev",
                "build": "build-1",
            },
        ),
        (
            "1.2-dev+build-1",
            {
                "major": 1,
                "minor": 2,
                "patch": 0,
                "prerelease": "dev",
                "build": "build-1",
            },
        ),
        (
            "1.2+build-1",
            {
                "major": 1,
                "minor": 2,
                "patch": 0,
                "prerelease": None,
                "build": "build-1",
            },
        ),
        (
            "1.2",
            {"major": 1, "minor": 2, "patch": 0, "prerelease": None, "build": None},
        ),
    ],
)
def test_validate_version_number(value, expected):
    assert schemas.VersionField().deserialize(value) == expected
